==> apt-cacher-ng_0.4.6.orig.tar.gz: apt-cacher-ng-0.4.6/source/reportgen.cc <==

#include "reportgen.h"

#include "meta.h"
#include "acbuf.h"
#include "acfg.h"
#include "aclogger.h"

#include "lockable.h"
#include "filereader.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <glob.h>

#include <vector>
#include <iostream>

#include <stdio.h>

using namespace MYSTD;

reportgen::reportgen(int fd, bool bVerb) : maintenance(fd), m_bVerbose(bVerb)
{
}

void reportgen::Run(const string &s)
{
	SendDecoration(true);
	GenReport(s);
	SendDecoration(false);
}

#define nRatWid 12

void reportgen::GenReport(const string & sCmd)
{
	filereader reader;
	string sTempFile=acfg::confdir+sPathSep+"report.html";
	if(!reader.OpenFile(sTempFile))
	{
		SendMsg(string("Unable to open template file, ")+sTempFile);
		return;
	}
	next_line:
	string sLine;
	//double factor;
	//int nRatWid(200);
	vector<char> buf(4000);
	
	while(reader.GetOneLine(sLine))
	{
		SetStyle(sLine);
		StrSubst(sLine, "$SERVERIP", GetHostname());
		
		tStrPos pos=0, posPrev=0;
		while(true)
		{
			pos=sLine.find('@', posPrev);
			if (pos==stmiss)
			{
				SendMsg(sLine.substr(posPrev)+sCRLF);
				break;
			}
			SendMsg(sLine.substr(posPrev, pos-posPrev));
			pos++;
			if (pos>=sLine.length())
			{
				SendMsg("<font size=+2 color=red>Template error detected</font>");
				return;
			}
			switch (sLine[pos])
			{
			case ('H'):
			{
				gethostname(&buf[0], buf.size());
				SendMsg(&buf[0]);
				break;
			}
			/* bloat
			case ('F'):
			{
				factor=atof(sLine.c_str()+pos+1);
				if(FP_NORMAL!=fpclassify(factor))
					factor=1.0;
				break;
			}
			*/
			case ('W'):
			{
				/* stupid bars disabled, define as constant now
							
							nRatWid=atoi(sLine.c_str()+pos+1);
							*/
				goto next_line;
			}
			
			case('A'):
			{
				if(acfg::exfailabort)
					SendMsg("checked");
				break;;
			}
			case ('T'):
			{
				if(!m_bVerbose)
				{
				SendMsg("<tr><td class=\"colcont\"><form action=\"#top\">"
						"<input type=\"submit\" name=\"doCount\" value=\"Count Data\"></form>"
						"</td><td class=\"colcont\" colspan=8 valign=top><font size=-2>"
						"<i>Not calculated, click \"Count data\"</i></font></td></tr>");
				}
				else
				{
					vector<aclog::tRowData> data;
					aclog::GetStats(data);
					for (vector<aclog::tRowData>::iterator it=data.begin(); it
							!=data.end(); it++)
					{
						
						unsigned long reqMax = max(it->reqIn, it->reqOut);
						uint64_t dataMax =  max(it->byteIn, it->byteOut);
						
						if( 0 == dataMax || 0 == reqMax)
							continue;
												
						char tbuf[50];
						size_t zlen(0);
						ctime_r(&it->from, tbuf);
						struct tm *tmp=localtime(&it->from);
						if (!tmp)
							goto time_error;
						zlen=strftime(tbuf, sizeof(tbuf), TIMEFORMAT, tmp);
						if ( !zlen)
							goto time_error;

						if (it->from!=it->to)
						{
							tmp=localtime(&it->to);
							if (!tmp)
								goto time_error;
							if ( 0 == strftime(tbuf+zlen, sizeof(tbuf)-zlen,
									" - " TIMEFORMAT, tmp))
								goto time_error;
						}
						snprintf(&buf[0], buf.size(), 
								"<tr bgcolor=\"white\">"

								"<td class=\"colcont\">%s</td>"

								"<td class=\"coltitle\"><span>&nbsp;</span></td>"
								
								"<td class=\"colcont\">%lu (%2.2f%%)</td>"
								"<td class=\"colcont\">%lu (%2.2f%%)</td>"
								"<td class=\"colcont\">%lu</td>"

								"<td class=\"coltitle\"><span>&nbsp;</span></td>"
								
								"<td class=\"colcont\">%2.2f MiB (%2.2f%%)</td>"
								"<td class=\"colcont\">%2.2f MiB (%2.2f%%)</td>"
								"<td class=\"colcont\">%2.2f MiB</td>"
								"</tr>"
								, 
								tbuf,
								reqMax - it->reqIn, double(reqMax - it->reqIn) / reqMax * 100, // hitcount
								it->reqIn, double(it->reqIn) / reqMax * 100, // misscount
								reqMax,
								
								double(dataMax - it->byteIn)/1048576, double(dataMax - it->byteIn)/dataMax*100, // hitdata
								double(it->byteIn)/1048576, double(it->byteIn)/dataMax*100, // missdata
								double(dataMax)/1048576
								
								); //, int(it->ratioSent*nRatWid), int((1.0-it->ratioSent)*nRatWid));
						SendMsg(&buf[0]);
						continue;
						time_error: SendMsg(" Invalid time value detected, check the stats database. ");
					}
				}
				goto next_line;
			}
			default:
				//out+="<b>Unknown template variable</b>";
				SendMsg(string("@")+sLine[pos]);
				break;
			}
			posPrev=pos+1;
		}

	}
}

==> apt-cacher-ng_0.5.1.orig.tar.gz: apt-cacher-ng-0.5.1/source/reportgen.cc <==
#if 0
#include "reportgen.h"

#include "meta.h"
#include "acbuf.h"
#include "acfg.h"
#include "aclogger.h"

#include "lockable.h"
#include "filereader.h"

#include "fileio.h"
#include <unistd.h>

#include <vector>
#include <iostream>

using namespace MYSTD;

reportgen::reportgen(int fd, bool bVerb) : maintenance(fd), m_bVerbose(bVerb)
{
}

void reportgen::Run(const string &s)
{
	SendDecoration(true);
	GenReport(s);
	SendDecoration(false);
}

#define nRatWid 12

void reportgen::GenReport(const string & sCmd)
{
	filereader reader;
	string sTempFile=acfg::confdir+sPathSep+"report.html";
	if(!reader.OpenFile(sTempFile))
	{
		SendChunk(string("Unable to open template file, ")+sTempFile);
		return;
	}
	next_line:
	string sLine;
	//double factor;
	//int nRatWid(200);
	vector<char> buf(4000);
	
	while(reader.GetOneLine(sLine))
	{
		SetStyle(sLine);
		StrSubst(sLine, "$SERVERIP", GetHostname());
		
		tStrPos pos=0, posPrev=0;
		while(true)
		{
			pos=sLine.find('@', posPrev);
			if (pos==stmiss)
			{
				SendChunk(sLine.substr(posPrev)+sCRLF);
				break;
			}
			SendChunk(sLine.substr(posPrev, pos-posPrev));
			pos++;
			if (pos>=sLine.length())
			{
				SendChunk("<font size=+2 color=red>Template error detected</font>");
				return;
			}
			switch (sLine[pos])
			{
			case ('H'):
			{
				gethostname(&buf[0], buf.size());
				SendChunk(&buf[0]);
				break;
			}
			/* bloat
			case ('F'):
			{
				factor=atof(sLine.c_str()+pos+1);
				if(FP_NORMAL!=fpclassify(factor))
					factor=1.0;
				break;
			}
			*/
			case ('W'):
			{
				/* stupid bars disabled, define as constant now
							
							nRatWid=atoi(sLine.c_str()+pos+1);
							*/
				goto next_line;
			}
			
			case('A'):
			{
				if(acfg::exfailabort)
					SendChunk("checked");
				break;;
			}
			case ('T'):
			{
				if(!m_bVerbose)
				{
				SendChunk("<tr><td class=\"colcont\"><form action=\"#top\">"
						"<input type=\"submit\" name=\"doCount\" value=\"Count Data\"></form>"
						"</td><td class=\"colcont\" colspan=8 valign=top><font size=-2>"
						"<i>Not calculated, click \"Count data\"</i></font></td></tr>");
				}
				else
				{
					vector<aclog::tRowData> data;
					aclog::GetStats(data);
					for (vector<aclog::tRowData>::iterator it=data.begin(); it
							!=data.end(); it++)
					{
						
						unsigned long reqMax = max(it->reqIn, it->reqOut);
						uint64_t dataMax =  max(it->byteIn, it->byteOut);
						
						if( 0 == dataMax || 0 == reqMax)
							continue;
												
						char tbuf[50];
						size_t zlen(0);
						ctime_r(&it->from, tbuf);
						struct tm *tmp=localtime(&it->from);
						if (!tmp)
							goto time_error;
						zlen=strftime(tbuf, sizeof(tbuf), TIMEFORMAT, tmp);
						if ( !zlen)
							goto time_error;

						if (it->from!=it->to)
						{
							tmp=localtime(&it->to);
							if (!tmp)
								goto time_error;
							if ( 0 == strftime(tbuf+zlen, sizeof(tbuf)-zlen,
									" - " TIMEFORMAT, tmp))
								goto time_error;
						}
						snprintf(&buf[0], buf.size(), 
								"<tr bgcolor=\"white\">"

								"<td class=\"colcont\">%s</td>"

								"<td class=\"coltitle\"><span>&nbsp;</span></td>"
								
								"<td class=\"colcont\">%lu (%2.2f%%)</td>"
								"<td class=\"colcont\">%lu (%2.2f%%)</td>"
								"<td class=\"colcont\">%lu</td>"

								"<td class=\"coltitle\"><span>&nbsp;</span></td>"
								
								"<td class=\"colcont\">%2.2f MiB (%2.2f%%)</td>"
								"<td class=\"colcont\">%2.2f MiB (%2.2f%%)</td>"
								"<td class=\"colcont\">%2.2f MiB</td>"
								"</tr>"
								, 
								tbuf,
								reqMax - it->reqIn, double(reqMax - it->reqIn) / reqMax * 100, // hitcount
								it->reqIn, double(it->reqIn) / reqMax * 100, // misscount
								reqMax,
								
								double(dataMax - it->byteIn)/1048576, double(dataMax - it->byteIn)/dataMax*100, // hitdata
								double(it->byteIn)/1048576, double(it->byteIn)/dataMax*100, // missdata
								double(dataMax)/1048576
								
								); //, int(it->ratioSent*nRatWid), int((1.0-it->ratioSent)*nRatWid));
						SendChunk(&buf[0]);
						continue;
						time_error: SendChunk(" Invalid time value detected, check the stats database. ");
					}
				}
				goto next_line;
			}
			default:
				//out+="<b>Unknown template variable</b>";
				SendChunk(string("@")+sLine[pos]);
				break;
			}
			posPrev=pos+1;
		}

	}
}

#endif
